import java.util.ArrayList;

/**
 *  This class is the main class of the "World of Zuul" application. 
 *  "World of Zuul" is a very simple, text based adventure game.  Users 
 *  can walk around some scenery. That's all. It should really be extended 
 *  to make it more interesting!
 * 
 *  To play this game, create an instance of this class and call the "play"
 *  method.
 * 
 *  This main class creates and initialises all the others: it creates all
 *  rooms, creates the parser and starts the game.  It also evaluates and
 *  executes the commands that the parser returns.
 * 
 * @author  Michael Kolling and David J. Barnes
 * @version 1.0 (February 2002)
 */

public class Game 
{
    private Parser parser;
    private Player player;
    private Room outside;
    
   /**
    * Standard constructor
    */
    public Game() {
        parser = new Parser();
        outside = RoomMaker.createRooms();
        player = new Player(outside);
        player.getInventory().add(new Light("flashlight", 4));
        player.getInventory().add(new Item("cookie", 2));  
    }
   
 
    /**
     *  Main play routine.  Loops until end of play.
     */
    public void play() 
    {            
        printWelcome();

        // Enter the main command loop.  Here we repeatedly read commands and
        // execute them until the game is over.
                
        boolean finished = false;
        while (! finished) {
            Command command = parser.getCommand();
            finished = processCommand(command);
        }
        System.out.println("Thank you for playing.  Good bye.");
    }

    /**
     * Print out the opening message for the player.
     */
    private void printWelcome()
    {
        System.out.println();
        System.out.println("Welcome to the World of Zuul!");
        System.out.println("World of Zuul is a new, incredibly boring adventure game.");
        System.out.println("Type 'help' if you need help.");
        System.out.println();
        Room currentRoom = player.getCurrentRoom();
        currentRoom.onEntry(player);
        System.out.println(currentRoom.getLongDescription(player));
    }

    /**
     * Given a command, process (that is: execute) the command.
     * If this command ends the game, true is returned, otherwise false is
     * returned.
     */
    private boolean processCommand(Command command) {
        if(command.isUnknown()) {
            System.out.println("I don't know what you mean...");
            return false;
        }

        String commandWord = command.getCommandWord();
        
        if (commandWord.equals("quit")) {      
            return true;        
        } else if (commandWord.equals("help"))
            printHelp();
        else if (commandWord.equals("go")) {
            goRoom(command);
        } else if (commandWord.equals("inventory")) {
            printInventory(player);
        } else if (commandWord.equals("look")) {
            System.out.println(player.getCurrentRoom().getLongDescription(player));
        } else {
            player.getCurrentRoom().respond(command, player);
        }
        
        return false;     
    }

    /**
     * Print out some help information.
     * Here we print some stupid, cryptic message and a list of the 
     * command words.
     */
    private void printHelp() 
    {
        System.out.println("You are lost. You are alone. You wander");
        System.out.println("around at the university.");
        System.out.println();
        System.out.println("Your command words are:");
        parser.showCommands();
    }

    /** 
     * Try to go to one direction. If there is an exit, enter the new
     * room, otherwise print an error message.
     * pld: if getExit() returns null, we can't go that way, and getExit()
     * will print why. Returning a null signals to goRoom *not* to
     * print the description. 
     * Note that there may be cases where getExit returns the same room as the one we're in;
     * the usual understanding of such a situation is that we somehow just returned
     * to the room we were in (as via a teleporter or in a maze), but it wasn't
     * "obviously" the same room.
     */
    private void goRoom(Command command) 
    {
        if(!command.hasSecondWord()) {
            // if there is no second word, we don't know where to go...
            System.out.println("Go where?");
            return;
        }

        String direction = command.getSecondWord();

        // Try to leave current room.
        Room nextRoom = player.getCurrentRoom().getExit(direction, player);

        if (nextRoom == null) {
            // Make getExit be responsible for printing out the reason why not - pld
            //System.out.println("There is no door!");
        } else {
            Room currentRoom = nextRoom;
            player.setCurrentRoom(currentRoom);
            currentRoom.onEntry(player);
            System.out.println(currentRoom.getLongDescription(player));
        }
    }

    /** 
     * "Quit" was entered. Check the rest of the command to see
     * whether we really quit the game. Return true, if this command
     * quits the game, false otherwise.
     */
    private boolean quit(Command command) 
    {
        if(command.hasSecondWord()) {
            System.out.println("Quit what?");
            return false;
        }
        else
            return true;  // signal that we want to quit
    }
    
    /**
     * printInventory prints the contents of "inventory"
     */
    private void printInventory(Player p) {
        ArrayList<Item> inventory = p.getInventory();
        if (inventory.size() == 0) {
            System.out.println("you are not carrying anything");
        } else {
            System.out.print("You have the following:");
            for (int n = 0; n < inventory.size(); n++) {
                Item item = (Item) inventory.get(n);
                System.out.print(" " + item.getDesc());
            }
            System.out.println();
            System.out.println("Total weight: " + totalWeight(inventory));
        }
    }
    
    /**
     * findByName: given a string and an ArrayList of Items, 
     * find the Item with the matching name, or else return null
     */
    static public Item findByName(String s, ArrayList<Item> L) {
        int n=0;
        while (n < L.size()) {
            Item i =  L.get(n);
            if (s.equals(i.getDesc())) 
                return i;
            n++;
        }
        return null;    // not found above
    }
   
   /**
     * totalWeight: given an ArrayList of Items, returns total of weights
     */
    public static int totalWeight(ArrayList<Item> L) {
        int n=0;
        int sum = 0;
        while (n < L.size()) {
            Item i =  L.get(n);
            sum += i.getWeight();
            n++;
        }
        return sum;    // not found above
    }
      
    /* */
}
